% Configuration of the problem: num. nodes, initial risk and node coordinates
clear;
n = 3000;
x0 = rand(n,1);
coor = rand(n,2);

% Random graph
% A = genrandom(n);

% RGG graph
A = genrgg(n,coor,0.05);

% RGG graph with long-range edge
% A = genrgg_long(n,coor,0.05);

% Preferential attachment
% A = genpref(n);

% plot the network
p = plot(graph(A),'XData',coor(:,1),'YData',coor(:,2),'NodeCData',x0,'EdgeColor','#cccccc');
colormap(flipud(hot))
colorbar
axis([0 1 0 1])
[r,c] = find(A);
csvwrite("covid.csv",[r c]);

% Creation of the grid with initial risk values
grid_size = 10;
loc = ceil(coor * grid_size);
status = zeros(n,grid_size,grid_size);
for i = 1:n
    status(i,loc(i,2), loc(i,1)) = x0(i);
end

% Consensus configuration
L = diag(sum(A)) - A;
eps = 1 / max(max(diag(sum(A)))) * 0.99;
P = eye(n) - eps * L;

% Consensus process
iter = 1500;
x = reshape(status,[n,grid_size * grid_size]);
% just to show the progress of the consensus
% the lines marqued with (C) can be commented or removed
output = zeros(n,iter); % (C)          
for i = 2:iter
    x = P * x;
    output(:,i) = sum(x,2); % (C)
end
plot(output'); % (C)

% Show the final map
map = reshape(x(1,:,:),[grid_size,grid_size]);
[gx,gy] = meshgrid(1:grid_size);

figure
subplot(1,2,1);
surface(gx,gy,map,'FaceColor','inter')
colormap(flipud(hot))
subplot(1,2,2);
surface(gx,gy,map)
colormap(flipud(hot))
colorbar